@@ -1,11 +1,11 @@
-# pylint: disable=broad-except,invalid-name,protected-access,consider-using-ternary,too-many-locals,too-many-arguments
+# pylint: disable=broad-except,invalid-name,protected-access,consider-using-ternary
 """
 Set of utility functions that use spaCy to perform named entity recognition
 """
 
 import logging
+import multiprocessing
 from datetime import datetime
-from multiprocessing import Pool, current_process
 from pathlib import Path
 from typing import Callable, Dict, Iterable, List, Optional, Tuple
 
@@ -21,8 +21,12 @@ from libratom.lib.constants import (
     BodyType,
 )
 from libratom.lib.core import get_cached_spacy_model
+from libratom.lib.headers import (
+    get_header_field_type_mapping,
+    populate_header_field_types,
+)
 from libratom.lib.utils import cleanup_message_body
-from libratom.models import Attachment, Entity, FileReport, Message
+from libratom.models import Attachment, Entity, FileReport, HeaderField, Message
 
 logger = logging.getLogger(__name__)
 
@@ -58,16 +62,6 @@ def process_message(
             body, body_type, RATOM_SPACY_MODEL_MAX_LENGTH
         )
 
-        # https://github.com/explosion/spaCy/issues/6662#issuecomment-753889021
-        if (
-            spacy_model_name.endswith("_trf")
-            and current_process().name != "MainProcess"
-        ):
-            # pylint: disable=all
-            import torch
-
-            torch.set_num_threads(1)
-
         spacy_model = get_cached_spacy_model(spacy_model_name)
         doc = spacy_model(message_body)
         res["entities"] = [(ent.text, ent.label_) for ent in doc.ents]
@@ -101,9 +95,9 @@ def extract_entities(
     """
 
     # Confirm environment settings
-    for key, value in globals().items():
-        if key.startswith("RATOM_"):
-            logger.debug(f"{key}: {value}")
+    for setting_name, setting_value in globals().items():
+        if setting_name.startswith("RATOM_"):
+            logger.debug(f"{setting_name}: {setting_value}")
 
     # Default progress callbacks to no-op
     processing_update_progress = processing_progress_callback or (lambda *_, **__: None)
@@ -112,8 +106,20 @@ def extract_entities(
     # Load the file_report table for local lookup
     _file_reports = session.query(FileReport).all()  # noqa: F841
 
+    # Add header field type table
+    if include_message_contents:
+        populate_header_field_types(session)
+
+    # Cache header field types into local mapping,
+    # empty if header field type table was not created
+    header_field_type_mapping = get_header_field_type_mapping(session)
+
     # Start of multiprocessing
-    with Pool(processes=jobs, initializer=worker_init) as pool:
+    ctx = multiprocessing.get_context(
+        "spawn" if spacy_model_name.endswith("_trf") else None
+    )  # https://github.com/explosion/spaCy/issues/6662
+
+    with ctx.Pool(processes=jobs, initializer=worker_init) as pool:
 
         logger.debug(f"Starting pool with {pool._processes} processes")
 
@@ -142,6 +148,7 @@ def extract_entities(
 
                 if error:
                     logger.info(
+                        # pylint: disable=consider-using-f-string
                         "Skipping message {message_id} from {filepath}".format(**res)
                     )
                     logger.debug(error)
@@ -183,6 +190,28 @@ def extract_entities(
                     ]
                 )
 
+                # Record header fields
+                if include_message_contents:
+                    header_fields = []
+
+                    for line in (res.get("headers") or "").splitlines():
+                        try:
+                            header_name, header_value = line.split(":", maxsplit=1)
+                        except ValueError:
+                            continue
+                        if header_field_type := header_field_type_mapping.get(
+                            header_name.lower()
+                        ):
+                            header_fields.append(
+                                HeaderField(
+                                    header_field_type=header_field_type,
+                                    value=header_value,
+                                    message=message,
+                                )
+                            )
+
+                    session.add_all(header_fields)
+
                 # Record entities info
                 for entity in entities:
                     new_entities.append(
